@@ -18,6 +18,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+
+#include "config.h" // For HAVE_STRPTIME
+
 #include "glomconversions.h"
 #include <glom/libglom/connectionpool.h>
 #include <glom/libglom/utils.h>
@@ -30,11 +33,15 @@
 #include <ctime>     // for struct tm
 #include <iostream>   // for cout, endl
 #include <iomanip>
-
+#include <string.h> // for strlen, memset, strcmp
 
 namespace Glom
 {
 
+// Increase the number of digits (even before the decimal point) we can 
+// have until it uses the awkward e syntax. The default seems to be 7
+static const int STRINGSTREAM_PRECISION_DEFAULT = 15;
+
 Glib::ustring Conversions::format_time(const tm& tm_data)
 {
   return format_time( tm_data, std::locale("") /* the user's current locale */ ); //Get the current locale.
@@ -48,7 +55,7 @@ Glib::ustring Conversions::format_time(const tm& tm_data, const std::locale& loc
     return format_tm(tm_data, locale, "%T" /* I see no iso-format time in the list, but this looks like the standard C-locale format. murrayc*/);
   }
   else
-    return format_tm(tm_data, locale, _("%X") /* time */);
+    return format_tm(tm_data, locale, "%X" /* time */);
 }
 
 Glib::ustring Conversions::format_date(const tm& tm_data)
@@ -58,10 +65,25 @@ Glib::ustring Conversions::format_date(const tm& tm_data)
 
 #define GLOM_NON_TRANSLATED_LOCALE_DATE_FORMAT "%x"
 
-static const char* glom_get_locale_date_format()
+// The % format to use to print and interpret dates, with 4-digit years:
+static const gchar* c_locale_date_format = 0;
+
+static inline const char* glom_get_locale_date_format()
 {
-  /* TRANSLATORS: Please only translate this string if you know that strftime() shows only 2 year digits when using format "x". We want to always display 4 year digits. For instance, en_GB should translate it to "%d/%m/%Y". To discover if your locale has this problem, try the testdateput_allformats.cc test case in http://bugzilla.gnome.org/show_bug.cgi?id=334648. Thanks. */
-  return _("%x");
+  if(!c_locale_date_format)
+  {
+    /* TRANSLATORS: Please only translate this string if you know that strftime() 
+     * shows only 2 year digits when using format "x". We want to always display 
+     * 4 year digits. For instance, en_GB should translate it to "%d/%m/%Y". 
+     * Glom will show a warning in the terminal at startup if this is necessary 
+     * and default to %d/%m/%Y" if it detects a problem, but that might not be 
+     * correct for your locale.
+     * Thanks.
+     */
+    c_locale_date_format = _("%x");
+  }
+
+  return c_locale_date_format;
 }
 
 Glib::ustring Conversions::format_date(const tm& tm_data, const std::locale& locale, bool iso_format)
@@ -75,9 +97,84 @@ Glib::ustring Conversions::format_date(const tm& tm_data, const std::locale& loc
   }
 }
 
+bool Conversions::sanity_check_date_parsing()
+{
+  //A date that is really really the date that we mean:
+  tm the_c_time;
+  memset(&the_c_time, 0, sizeof(the_c_time));
+
+  //We mean 22nd November 2008:
+  the_c_time.tm_year = 2008 - 1900; //C years start are the AD year - 1900. So, 01 is 1901.
+  the_c_time.tm_mon = 11 - 1; //C months start at 0.
+  the_c_time.tm_mday = 22; //starts at 1
+
+  //Get the current locale's text representation:
+  const Glib::ustring date_text = format_date(the_c_time);
+  //std::cout << "DEBUG: 22nd November 2008 in this locale has this text represention: " << date_text << std::endl;
+
+  //Try to parse it:
+  bool success = false;
+  tm parsed_date = parse_date(date_text, success);
+
+  if(success)
+  {
+    //std::cout << "  DEBUG: parsed date: year=" << parsed_date.tm_year + 1900 << ", month=" << parsed_date.tm_mon + 1 << ", day=" << parsed_date.tm_mday << std::endl;
+  }
+
+  if(!success ||
+     parsed_date.tm_year != the_c_time.tm_year || 
+     parsed_date.tm_mon != the_c_time.tm_mon || 
+     parsed_date.tm_mday != the_c_time.tm_mday)
+  {
+    //Note to translators: If you see this error in the terminal at startup then you need to translate the %x elsewhere.
+    std::cerr << _("ERROR: sanity_check_date_parsing(): Sanity check failed: Glom could not parse a date's text representation that it generated itself, in this locale.") << std::endl;
+
+    //If translators cannot be relied upon to do this, maybe we should default to "%d/%m/%Y" when "%x" fails this test.
+
+    return false;
+  } 
+     
+  return true;
+}
+
+bool Conversions::sanity_check_date_text_representation_uses_4_digit_years()
+{
+  //A date that is really really the date that we mean:
+  tm the_c_time;
+  memset(&the_c_time, 0, sizeof(the_c_time));
+
+  //We mean 22nd November 2008:
+  the_c_time.tm_year = 2008 - 1900; //C years start are the AD year - 1900. So, 01 is 1901.
+  the_c_time.tm_mon = 11 - 1; //C months start at 0.
+  the_c_time.tm_mday = 22; //starts at 1
+
+  //Get the current locale's text representation:
+  const Glib::ustring date_text = format_date(the_c_time);
+  std::cout << "DEBUG: 22nd November 2008 in this locale has this text represention: " << date_text << std::endl;
+
+  //See if the year appears in full in that date.
+  //There are probably some locales for which this fails.
+  //Please tell us if there are.
+  const Glib::ustring::size_type pos = date_text.find("2008");
+  if(pos == Glib::ustring::npos)
+  {
+    //Note to translators: If you see this error in the terminal at startup then you need to translate the %x elsewhere.
+    std::cerr << _("ERROR: sanity_check_date_text_represenation_uses_4_digit_year(): Sanity check failed: Glom does not seem to use 4 digits to display years in a date's text representation, in this locale. Defaulting to dd/mm/yyyy though this might be incorrect for your locale. This needs attention from a translator. Please file a bug - see http://www.glom.org") << std::endl;
+
+    //Do not depend on translators to do what we ask.
+    //Default to a common format, though this would be incorrect in some 
+    //locales, such as German.
+    c_locale_date_format = "%d/%m/%Y";
+
+    return false;
+  }
+
+  return true;
+}
+
 
 Glib::ustring Conversions::format_tm(const tm& tm_data, const std::locale& locale, const char* format)
-{
+{   
   //This is based on docs found here:
   //http://www.roguewave.com/support/docs/sourcepro/stdlibref/time-put.html
 
@@ -95,6 +192,7 @@ Glib::ustring Conversions::format_tm(const tm& tm_data, const std::locale& local
   tp.put(the_stream /* iter to beginning of stream */, the_stream, ' ' /* fill */, &tm_data, format, format + strlen(format) /* 'E' */ /* use locale's alternative format */);
 
   Glib::ustring text = the_stream.str();
+  //std::cout << "DEBUG: format_tm(): result from tp.put: " << text << std::endl;
 
   if(locale == std::locale("") /* The user's current locale */)
   {
@@ -109,6 +207,7 @@ Glib::ustring Conversions::format_tm(const tm& tm_data, const std::locale& local
 #endif
   }
 
+  //std::cout << "DEBUG: format_tm(): returning: " << text << std::endl;
   return text; //TODO: Use something like Glib::locale_to_utf8()?
 
   /*
@@ -169,6 +268,29 @@ Glib::ustring Conversions::get_text_for_gda_value(Field::glom_field_type glom_ty
   return get_text_for_gda_value(glom_type, value, std::locale("") /* the user's current locale */, numeric_format); //Get the current locale.
 }
 
+double Conversions::get_double_for_gda_value_numeric(const Gnome::Gda::Value& value)
+{
+  if(value.get_value_type() != GDA_TYPE_NUMERIC)
+  {
+    std::cerr << "Conversions::get_double_for_gda_value_numeric(): expected NUMERIC but GdaValue type is: " << g_type_name(value.get_value_type()) << std::endl;
+    return 0;
+  }
+
+  const GdaNumeric* gda_numeric = value.get_numeric();
+  std::string text_in_c_locale;
+  if(gda_numeric && gda_numeric->number) //A char* - I assume that it formatted as per the C locale. murrayc. TODO: Do we need to look at the other fields?
+    text_in_c_locale = gda_numeric->number; //What formatting does this use?
+
+  //Get an actual numeric value, so we can get a locale-specific text representation:
+  std::stringstream the_stream;
+  the_stream.imbue( std::locale::classic() ); //The C locale.
+  the_stream.str(text_in_c_locale); //Avoid using << because Glib::ustring does implicit character conversion with that.
+
+  double number = 0;
+  the_stream >> number;
+  return number;
+}
+
 Glib::ustring Conversions::get_text_for_gda_value(Field::glom_field_type glom_type, const Gnome::Gda::Value& value, const std::locale& locale, const NumericFormat& numeric_format, bool iso_format)
 {
   if(value.is_null()) //The type can be null for any of the actual field types.
@@ -211,20 +333,15 @@ Glib::ustring Conversions::get_text_for_gda_value(Field::glom_field_type glom_ty
 
     return format_time(the_c_time, locale, iso_format);
   }
-  else if( (glom_type == Field::TYPE_NUMERIC) && (value.get_value_type() == GDA_TYPE_NUMERIC))
+  else if(glom_type == Field::TYPE_NUMERIC)
   {
-    const GdaNumeric* gda_numeric = value.get_numeric();
-    std::string text_in_c_locale;
-    if(gda_numeric && gda_numeric->number) //A char* - I assume that it formatted as per the C locale. murrayc. TODO: Do we need to look at the other fields?
-      text_in_c_locale = gda_numeric->number; //What formatting does this use?
+    if(value.get_value_type() != GDA_TYPE_NUMERIC)
+    {
+      std::cerr << "Conversions::get_text_for_gda_value(): glom field type is NUMERIC but GdaValue type is: " << g_type_name(value.get_value_type()) << std::endl;
+      return value.to_string();
+    }
 
-    //Get an actual numeric value, so we can get a locale-specific text representation:
-    std::stringstream the_stream;
-    the_stream.imbue( std::locale::classic() ); //The C locale.
-    the_stream.str(text_in_c_locale); //Avoid using << because Glib::ustinrg does implicit character conversion with that.
-
-    double number = 0;
-    the_stream >> number;
+    const double number = get_double_for_gda_value_numeric(value);
 
     //Get the locale-specific text representation, in the required format:
     std::stringstream another_stream;
@@ -243,7 +360,15 @@ Glib::ustring Conversions::get_text_for_gda_value(Field::glom_field_type glom_ty
       if(numeric_format.m_decimal_places_restricted)
       {
         another_stream << std::fixed;
-        another_stream << std::setprecision(numeric_format.m_decimal_places); //precision means number of decimal places when using std::fixed.
+
+        //precision means number of decimal places when using std::fixed:
+        another_stream << std::setprecision(numeric_format.m_decimal_places);
+      }
+      else
+      {
+        //Increase the number of digits (even before the decimal point) we can 
+        //have until it uses the awkward e syntax. The default seems to be 7.
+        another_stream << std::setprecision(STRINGSTREAM_PRECISION_DEFAULT);
       }
 
       if(!(numeric_format.m_currency_symbol.empty()))
@@ -302,6 +427,7 @@ Gnome::Gda::Value Conversions::parse_value(double number)
   std::stringstream clocale_stream;
   clocale_stream.imbue( std::locale::classic() ); //The C locale.
   clocale_stream << number;
+  clocale_stream << std::setprecision(STRINGSTREAM_PRECISION_DEFAULT); //Avoid the e syntax. Normally this happens afer 7 digits, with loss of precision. TODO: Handle more.
   Glib::ustring number_canonical_text = clocale_stream.str(); //Avoid << because it does implicit character conversion (though that might not be a problem here. Not sure). murrayc
 
   //TODO: What about the precision and width values?
@@ -323,7 +449,7 @@ Gnome::Gda::Value Conversions::parse_value(Field::glom_field_type glom_type, con
 
 Gnome::Gda::Value Conversions::parse_value(Field::glom_field_type glom_type, const Glib::ustring& text, const NumericFormat& numeric_format, bool& success, bool iso_format)
 {
-  std::locale the_locale = (iso_format ? std::locale::classic() :  std::locale("") /* The user's current locale */);
+  const std::locale the_locale = (iso_format ? std::locale::classic() :  std::locale("") /* The user's current locale */);
 
   //Put a NULL in the database for empty dates, times, and numerics, because 0 would be an actual value.
   //But we use "" for strings, because the distinction between NULL and "" would not be clear to users.
@@ -383,7 +509,7 @@ Gnome::Gda::Value Conversions::parse_value(Field::glom_field_type glom_type, con
 
     //Try to parse the inputted number, according to the current locale.
     std::stringstream the_stream;
-    the_stream.imbue( the_locale ); //Parse it as per the current locale.
+    the_stream.imbue( the_locale ); //Parse it as per the specified locale.
     the_stream.str(text_to_parse); //Avoid << because it does implicit character conversion (though that might not be a problem here. Not sure). murrayc
     double the_number = 0;
     the_stream >> the_number;  //TODO: Does this throw any exception if the text is an invalid time?
@@ -415,12 +541,12 @@ Gnome::Gda::Value Conversions::parse_value(Field::glom_field_type glom_type, con
   }
   else if(glom_type == Field::TYPE_IMAGE)
   {
-    //We assume that the text is the same (escaped text) format that we use in the document when saving examples.
+    //We assume that the text is the same (escaped text) format that we use in the document when saving images:
     //(The SQL format).
     Gnome::Gda::Value result;
 
     size_t buffer_binary_length = 0;
-    guchar* buffer_binary =  Glom_PQunescapeBytea((const guchar*)text.c_str() /* must be null-terminated */, &buffer_binary_length); //freed by us later.
+    guchar* buffer_binary = Glom_PQunescapeBytea((const guchar*)text.c_str() /* must be null-terminated */, &buffer_binary_length); //freed by us later.
     if(buffer_binary)
     {
       result.set(buffer_binary, buffer_binary_length);
@@ -492,7 +618,7 @@ tm Conversions::parse_date(const Glib::ustring& text, const std::locale& locale,
   }
   else
   {
-    std::cout << "DEBUG: Skipping std::time_get<>  because it is incapable of parsing 4-digit years in the current locale." << std::endl;
+    //std::cout << "DEBUG: Skipping std::time_get<>  because it is incapable of parsing 4-digit years in the current locale." << std::endl;
   }
 
   if(!skip_time_get && err != std::ios_base::failbit)
@@ -574,6 +700,7 @@ tm Conversions::parse_time(const Glib::ustring& text, bool& success)
 
 tm Conversions::parse_time(const Glib::ustring& text, const std::locale& locale, bool& success)
 {
+  //std::cout << "parse_time(): text=" << text << std::endl;
   //The sequence of statements here seems to be very fragile. If you move things then it stops working.
 
   //return parse_tm(text, locale, 'X' /* time */);
@@ -615,17 +742,62 @@ tm Conversions::parse_time(const Glib::ustring& text, const std::locale& locale,
     success = true;
     return the_c_time;
   }
-  else
+  
+#ifdef HAVE_STRPTIME
+  //Fall back to strptime():
+  //It seems to be well known that time_get<> can parse much less than what time_put<> can generate:
+  // http://www.open-std.org/jtc1/sc22/wg21/docs/papers/2006/n2070.html
+  //
+  //Try various formats:
+  
+  memset(&the_c_time, 0, sizeof(the_c_time));
+  char* lastchar = strptime(text.c_str(), "%r" /* 12-hour clock time using the AM/PM notation */, &the_c_time);
+  if(lastchar)
   {
-    //std::cout << "  tg.get_time() failed" << text << std::endl;
-
-    //tm blank_time = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
-    tm blank_time ;
-    memset(&blank_time , 0, sizeof(blank_time ));
-
-    success = false;
-    return blank_time;
+    success = true;
+    return the_c_time;
   }
+  
+  memset(&the_c_time, 0, sizeof(the_c_time));
+  lastchar = strptime(text.c_str(), "%X" /* The time, using the locale's time format. */, &the_c_time);
+  if(lastchar)
+  {
+    //std::cout << "DEBUG: parse_time(): %X: text=" << text << " was parsed as: hour=" << the_c_time.tm_hour << ", min=" << the_c_time.tm_min  << ", sec=" << the_c_time.tm_sec << std::endl;
+    success = true;
+    return the_c_time;
+  }
+  
+  //Note: strptime() with "%OI" parses "01:00 PM" incorrectly as 01:00, though it claims to parse successfully.
+  
+  memset(&the_c_time, 0, sizeof(the_c_time));
+  lastchar = strptime(text.c_str(), "%c" /* alternative 12-hour clock */, &the_c_time);
+  if(lastchar)
+  {
+    //std::cout << "DEBUG: parse_time(): %c: text=" << text << " was parsed as: hour=" << the_c_time.tm_hour << ", min=" << the_c_time.tm_min  << ", sec=" << the_c_time.tm_sec << std::endl;
+    success = true;
+    return the_c_time;
+  }
+  
+  //This seems to be the only one that can parse "01:00 PM":
+  memset(&the_c_time, 0, sizeof(the_c_time));
+  lastchar = strptime(text.c_str(), "%I : %M %p" /* 12 hours clock with AM/PM, without seconds. */, &the_c_time);
+  if(lastchar)
+  {
+    //std::cout << "DEBUG: parse_time(): %I : %M %p: text=" << text << " was parsed as: hour=" << the_c_time.tm_hour << ", min=" << the_c_time.tm_min  << ", sec=" << the_c_time.tm_sec << std::endl;
+    success = true;
+    return the_c_time;
+  }
+ 
+  //std::cout << "  DEBUG: strptime(%c) failed on text=" << text << std::endl;
+
+#endif // HAVE_STRPTIME
+  //Nothing worked:
+  //tm blank_time = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
+  tm blank_time ;
+  memset(&blank_time , 0, sizeof(blank_time ));
+
+  success = false;
+  return blank_time;
 }
 
 
@@ -673,7 +845,7 @@ bool Conversions::value_is_empty(const Gnome::Gda::Value& value)
     case(G_TYPE_STRING):
       return value.get_string().empty();
     default:
-      return false; //None of the other types can be empty. (An empty numeric, date, or time type shows up as a null.
+      return false; //None of the other types can be empty. (An empty numeric, date, or time type shows up as a NULL).
   }
 }
 
@@ -1010,7 +1182,7 @@ Glib::RefPtr<Gdk::Pixbuf> Conversions::get_pixbuf_for_gda_value(const Gnome::Gda
         {
           const Glib::Exception& ex = *error.get();        
 #endif
-          g_warning("ImageGlom::set_value(): PixbufLoader::write() failed: %s", ex.what().c_str());
+          g_warning("Conversions::get_pixbuf_for_gda_value(): PixbufLoader::write() failed: %s", ex.what().c_str());
         }
       }
 
