@@ -275,6 +275,11 @@ bool LayoutItem_Field::is_same_field(const sharedptr<const LayoutItem_Field>& fi
          (get_related_relationship_name() == field->get_related_relationship_name());
 }
 
+Glib::ustring LayoutItem_Field::get_sql_name(const Glib::ustring& parent_table) const
+{
+  return "\"" + get_sql_table_or_join_alias_name(parent_table) + "\".\"" + get_name() + "\"";
+}
+
 
 } //namespace Glom
 
