@@ -24,6 +24,9 @@
 #include "../../box_db_table.h"
 #include "dialog_fielddefinition.h"
 
+namespace Glom
+{
+
 class Box_DB_Table_Definition : public Box_DB_Table
 {
 public: 
@@ -40,7 +43,10 @@ protected:
 
   sharedptr<Field> get_field_definition(const Gtk::TreeModel::iterator& row);
 
-  virtual void change_definition(const sharedptr<const Field>& fieldOld, const sharedptr<const Field>& field);
+  sharedptr<Field> change_definition(const sharedptr<const Field>& fieldOld, const sharedptr<const Field>& field);
+
+  bool field_has_null_values(const sharedptr<const Field>& field);
+  bool field_has_non_unique_values(const sharedptr<const Field>& field);
 
   //Signal handlers:
   virtual void on_adddel_add(const Gtk::TreeModel::iterator& row);
@@ -50,18 +56,12 @@ protected:
 
   virtual void on_Properties_apply();
 
+  bool check_field_change(const sharedptr<const Field>& field_old, const sharedptr<const Field>& field_new);
+
   //Postgres needs some complex stuff:
-
-    /** @param not_extras If this is true, then do not set extra details, such as NOT NULL. You should do that later, when you are ready.
-   */
-  virtual void postgres_add_column(const sharedptr<const Field>& field, bool not_extras = false);
-  virtual void postgres_change_column(const sharedptr<const Field>& field_old, const sharedptr<const Field>& field);
+  virtual sharedptr<Field> postgres_change_column(const sharedptr<const Field>& field_old, const sharedptr<const Field>& field);
   virtual void postgres_change_column_type(const sharedptr<const Field>& field_old, const sharedptr<const Field>& field);
 
-  /** @param set_anyway If this is true, then set the extra details even if @field_old has the same properties.
-   */
-  virtual void postgres_change_column_extras(const sharedptr<const Field>& field_old, const sharedptr<const Field>& field, bool set_anyway = false);
-
   mutable AddDel_WithButtons m_AddDel; //mutable because its get_ methods aren't const.
 
   guint m_colName, m_colTitle, m_colType, m_colUnique, m_colPrimaryKey;
@@ -71,4 +71,6 @@ protected:
   type_vecFields m_vecFields;
 };
 
+} //namespace Glom
+
 #endif
