@@ -25,6 +25,10 @@
 #include <libglademm.h>
 #include <glom/libglom/data_structure/field.h>
 #include "../../base_db.h"
+#include <gtksourceviewmm/sourceview.h>
+
+namespace Glom
+{
 
 class Dialog_FieldCalculation
  : public Gtk::Dialog,
@@ -35,13 +39,14 @@ public:
   Dialog_FieldCalculation(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
   virtual ~Dialog_FieldCalculation();
 
-  virtual void set_field(const sharedptr<const Field>& field, const Glib::ustring& table_name);
-  virtual sharedptr<Field> get_field() const;
+  void set_field(const sharedptr<const Field>& field, const Glib::ustring& table_name);
+  sharedptr<Field> get_field() const;
 
 protected:
-  virtual void on_button_test();
+  void on_button_test();
+  bool check_for_return_statement(const Glib::ustring& calculation);
 
-  Gtk::TextView* m_text_view;
+  gtksourceview::SourceView* m_text_view;
   Gtk::Button* m_button_test;
   Gtk::Label* m_label_triggered_by;
 
@@ -49,4 +54,6 @@ protected:
   Glib::ustring m_table_name;
 };
 
+} //namespace Glom
+
 #endif //DIALOG_FIELDCALCULATION_H
